/**
 * Diese Datei ist Teil von „Lernstrategien“ 
 * (Dateiname: „mod_splernstrategien“), einer Plugin-Software 
 * für das moodle LMS.
 *
 * Das moodle LMS ist lizenziert mit der GNU General Public 
 * License Version 3. Aktuelle Lizenzhinweise zu moodle 
 * finden Sie unter: https://moodledev.io/general/license.
 *
 * Das Plugin ist lizenziert mit der GNU General Public 
 * License Version 3. Anzugebender Urheber des Plugins ist
 * die „metromorph softworks GmbH im Auftrag von ORCA.nrw“.
 * Die Lizenzierung umfasst nicht die Inhalte, die auf der
 * Anwenderoberfläche des Plugins angezeigt werden; diese sind 
 * ihrerseits CC BY-SA 4.0 lizenziert.
 *
 * Sie sollten eine Kopie der GNU-Lizenz zusammen mit diesem 
 * Programm erhalten haben. Sie finden sie auch unter 
 * https://www.gnu.org/licenses. Die CC-Lizenz finden Sie
 * unter https://creativecommons.org/licenses/by-sa/4.0/deed.de.
 */

/**
 * JS-module for drag and drop with a source and a target region.
 *
 * We use the sortable_list amd-module from moodle which depends on jquery.
 * Call the init-function with a source and a target class-names.
 * See http://componentlibrary.moodle.com/admin/tool/componentlibrary/docspage.php/moodle/javascript/sortable-list/
 * and https://github.com/moodle/moodle/blob/master/lib/amd/src/sortable_list.js
 * and template_265.mustache
 *
 * @module    mod_splernstrategien
 * @license   http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */

define(['jquery', 'core/sortable_list'],
    function($, SortableList) {
        /*
         * Get actual order of placement
         *
         */
        var getOrder = function(list, targetIds) {
            let order = Array.prototype.map.call(targetIds, (id) => {
                var ul = document.getElementById(id);
                if (ul.hasChildNodes() && ul.firstChild.nodeName == 'LI') {
                    return ul.getElementsByTagName('li')[0].id;
                } else {
                    return '';
                }
            });
            document.getElementById('result').value = order;
        };
        /*
         * Change source-target-behaviour of elements when dropping
         */
        var handleDrop = function(list, sourceList, className, targetClassName, targetIds) {
            // Change classes
            sourceList.classList.add(targetClassName);
            list.classList.remove(targetClassName);
            sourceList.classList.remove(className);
            list.classList.add(className);
            getOrder(list, targetIds);
        };

        /* Put the elements in the order of lastr try
         * in case of previous answer was wrong and we come by  repeatedly
         */
        var putInOrder = function(targetIds, order, className, targetClassName) {
            for (var i = 0; i < order.length; i++) {
                if (order[i] !== '') {
                    // Change classes on source and target
                    // append moved element to former place
                    var ul = document.getElementById(targetIds[i]);
                    var li = document.getElementById(order[i]);
                    li.parentElement.classList.add(targetClassName);
                    li.parentElement.classList.remove(className);
                    ul.appendChild(li);
                    ul.classList.remove(targetClassName);
                    ul.classList.add(className);
                }
            }
        };

        return {
            init: (className, targetClassName) => {
                var config = {
                    targetListSelector: 'ul.' + targetClassName
                };
                var targetIds = ['a', 'b', 'c', 'd', 'e', 'f', 'g', 'h'];

                // Put in previous order after wrong answer
                if (document.getElementById('result').value != '') {
                    var order = document.getElementById('result').value.split(',');
                    putInOrder(targetIds, order, className, targetClassName);
                }
                var list = new SortableList('ul.' + className, config);

                $('ul.' + className + ' > *').on(SortableList.EVENTS.DRAGEND, function(evt, info) {
                    handleDrop(info.targetList[0], info.sourceList[0], className, targetClassName, targetIds);
                });
                list.getElementName = function(element) {
                    return $.Deferred().resolve(element.attr('data-name'));
                };
                list.getDestinationName = function(parentElement) {
                    return $.Deferred().resolve(parentElement.attr('data-name'));
                };

            }
        };
    });
