<?php
/**
 * Diese Datei ist Teil von „Lernstrategien“ 
 * (Dateiname: „mod_splernstrategien“), einer Plugin-Software 
 * für das moodle LMS.
 *
 * Das moodle LMS ist lizenziert mit der GNU General Public 
 * License Version 3. Aktuelle Lizenzhinweise zu moodle 
 * finden Sie unter: https://moodledev.io/general/license.
 *
 * Das Plugin ist lizenziert mit der GNU General Public 
 * License Version 3. Anzugebender Urheber des Plugins ist
 * die „metromorph softworks GmbH im Auftrag von ORCA.nrw“.
 * Die Lizenzierung umfasst nicht die Inhalte, die auf der
 * Anwenderoberfläche des Plugins angezeigt werden; diese sind 
 * ihrerseits CC BY-SA 4.0 lizenziert.
 *
 * Sie sollten eine Kopie der GNU-Lizenz zusammen mit diesem 
 * Programm erhalten haben. Sie finden sie auch unter 
 * https://www.gnu.org/licenses. Die CC-Lizenz finden Sie
 * unter https://creativecommons.org/licenses/by-sa/4.0/deed.de.
 */

/**
 * Privacy subsystem for mod_splernstrategien.
 * 
 * @package   mod_splernstrategien
 * @category  privacy
 * @license   https://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */

namespace mod_splernstrategien\privacy;

use core_privacy\local\metadata\collection;
use core_privacy\local\request\contextlist;
use core_privacy\local\request\userlist;
use core_privacy\local\request\approved_contextlist;
use core_privacy\local\request\approved_userlist;
use core_privacy\local\request\writer;
use context;

defined('MOODLE_INTERNAL') || die();


class provider implements
    // This plugin does store personal user data.
    \core_privacy\local\metadata\provider,

    // This plugin currently implements the original plugin\provider interface.
    \core_privacy\local\request\plugin\provider,


    // This plugin is capable of determining which users have data within it.
    \core_privacy\local\request\core_userlist_provider,

    // This plugin has some sitewide user preferences to export.
    \core_privacy\local\request\user_preference_provider
{

    /**
     * Returns metadata.
     *
     * @param collection $collection The initialised collection to add items to.
     * @return collection A listing of user data stored through this system.
     */
    public static function get_metadata(collection $collection): collection
    {
        $collection->add_database_table(
            'splernstrategien_users',
            [
                'user_id'        => 'privacy:metadata:splernstrategien_users:userid',
                'course'        => 'privacy:metadata:splernstrategien_users:course',
                'laststate'     => 'privacy:metadata:splernstrategien_users:laststate',
                'nextstate'     => 'privacy:metadata:splernstrategien_users:nextstate',
                'epa_finished'  => 'privacy:metadata:splernstrategien_users:epa_finished',
                'epb_finished'  => 'privacy:metadata:splernstrategien_users:epb_finished',
                'wp_finished'   => 'privacy:metadata:splernstrategien_users:wp_finished',
                'delay'         => 'privacy:metadata:splernstrategien_users:delay',
                'timestamp'     => 'privacy:metadata:splernstrategien_users:timestamp'
            ],
            'privacy:metadata:splernstrategien_users'
        );
        $collection->add_database_table(
            'splernstrategien_answers',
            [
                'userid'    => 'privacy:metadata:splernstrategien_answers:userid',
                'course'    => 'privacy:metadata:splernstrategien_answers:course',
                'data'      => 'privacy:metadata:splernstrategien_answers:data',
            ],
            'privacy:metadata:splernstrategien_answers'
        );
        $collection->add_user_preference(
            'message_provider_mod_splernstrategien_trigger_enabled',
            'privacy:metadata:preference:message_provider_mod_splernstrategien_trigger_enabled'
        );

        return $collection;
    }

    /**
     * Get the list of contexts that contain user information for the specified user.
     *
     * @param int $userid The user to search.
     * @return contextlist $contextlist The contextlist containing the list of contexts used in this plugin.
     */
    public static function get_contexts_for_userid(int $userid): contextlist
    {
        $contextlist = new contextlist();
        $sql = "SELECT c.id
                FROM {context} c
                JOIN {course_modules} cm ON cm.id = c.instanceid 
                JOIN {modules} m ON m.id = cm.module
                JOIN {splernstrategien_users} u ON u.moodle_id = cm.id
                WHERE u.user_id = :userid AND c.contextlevel = :contextlevel AND m.name = :modname";
        $params = ['contextlevel' => CONTEXT_MODULE, 'modname' => 'splernstrategien', 'userid' => $userid];
        $contextlist->add_from_sql($sql, $params);

        return $contextlist;
    }

    /**
     * Get the list of users who have data within a context.
     *
     * @param   userlist    $userlist   The userlist containing the list of users who have data in this context/plugin combination.
     */
    
    public static function get_users_in_context(userlist $userlist)
    {
        $context = $userlist->get_context();

        if (!$context instanceof \context_module) {
            return;
        }

        $params = [
            'cmid'    => $context->instanceid,
            'modname' => 'splernstrategien',
        ];

        $sql = "SELECT u.user_id
                FROM {course_modules} cm
                JOIN {modules} m ON m.id = cm.module
                JOIN {splernstrategien_users} u ON u.moodle_id = cm.instance
                WHERE cm.id = :cmid AND m.name = :modname";
        $userlist->add_from_sql('userid', $sql, $params);
    }


    /**
     * Export all user preferences for the plugin.
     *
     * @param   int         $userid The userid of the user whose data is to be exported.
     */
    public static function export_user_preferences(int $userid)
    {
        $preference = get_user_preferences('message_provider_mod_splernstrategien_trigger_enabled', null, $userid);
        if (null !== $preference) {
            $desc = get_string('privacy:metadata:preference:message_provider_mod_splernstrategien_trigger_enabled', 'mod_splernstrategien');
            writer::export_user_preference('mod_splernstrategien', 'trigger_enabled', $preference, $desc);
        }
    }

    /**
     * Export all user data for the specified user, in the specified contexts.
     *
     * @param approved_contextlist $contextlist The approved contexts to export information for.
     */
    public static function export_user_data(approved_contextlist $contextlist)
    {
        global $DB;

        if (!count($contextlist)) {
            return;
        }
        $user = $contextlist->get_user();
        $userid = $user->id;
        $cmids = array_reduce($contextlist->get_contexts(), function ($carry, $context) {
            if ($context->contextlevel == CONTEXT_MODULE) {
                $carry[] = $context->instanceid;
            }
            return $carry;
        }, []);
        if (empty($cmids)) {
            return;
        }
        list($insql, $inparams) = $DB->get_in_or_equal($cmids, SQL_PARAMS_NAMED);
        $sql = "SELECT u.*,
                    a.data as answers               
                FROM {splernstrategien_users} u
                JOIN {splernstrategien_answers} a
                ON a.moodle_id = u.moodle_id AND a.user_id = u.user_id
                WHERE a.moodle_id $insql
                    AND a.user_id = :userid";
        $params = array_merge($inparams, ['userid' => $userid]);
        $recordset = $DB->get_recordset_sql($sql, $params);
        foreach ($recordset as $record) {
            $context = $record->moodle_id;
            $record->ans = json_decode($record->answers);
            unset($record->answers);
            writer::with_context(\context_module::instance($context))->export_data(
                [],
                $record
            );
        }
    }

    /**
     * Delete all data for all users in the specified context.
     *
     * @param context $context The specific context to delete data for.
     */
    public static function delete_data_for_all_users_in_context(context $context)
    {
        global $DB;

        if ($context->contextlevel != CONTEXT_MODULE) {
            return;
        }
        $cm = get_coursemodule_from_id('splernstrategien', $context->instanceid);
        if (!$cm) {
            return;
        }
        
        $DB->delete_records('splernstrategien_answers', ['moodle_id' => $cm->id]);
        $DB->delete_records('splernstrategien_users', ['moodle_id' => $cm->id]);
    }

    /**
     * Delete all user data for the specified user, in the specified contexts.
     *
     * @param approved_contextlist $contextlist The approved contexts and user information to delete information for.
     */
    public static function delete_data_for_user(approved_contextlist $contextlist)
    {
        global $DB;

        if (empty($contextlist->count())) {
            return;
        }
        $userid = $contextlist->get_user()->id;
        foreach ($contextlist->get_contexts() as $context) {
            $DB->delete_records('splernstrategien_users', ['moodle_id' => $context->instanceid, 'user_id' => $userid]);
            $DB->delete_records('splernstrategien_answers', ['moodle_id' => $context->instanceid, 'user_id' => $userid]);
        }
    }

    /**
     * Delete multiple users within a single context.
     *
     * @param   approved_userlist       $userlist The approved context and user information to delete information for.
     */
    public static function delete_data_for_users(approved_userlist $userlist)
    {
        global $DB;

        $context = $userlist->get_context();
       
        if ($context->contextlevel != CONTEXT_MODULE) {
            return;
        }
        $cm = get_coursemodule_from_id('splernstrategien', $context->instanceid);
        if (!$cm) {
            return;
        }
        $userids = $userlist->get_userids();

        list($usersql, $userparams) = $DB->get_in_or_equal($userids, SQL_PARAMS_NAMED);
        $select = "moodle_id = :id AND user_id $usersql";
        $params = ['id' => $cm->id] + $userparams;
        
        $DB->delete_records_select('splernstrategien_answers', $select, $params);
        $DB->delete_records_select('splernstrategien_users', $select, $params);
    } 
}
