<?php
/**
 * Diese Datei ist Teil von „Lernstrategien“ 
 * (Dateiname: „mod_splernstrategien“), einer Plugin-Software 
 * für das moodle LMS.
 *
 * Das moodle LMS ist lizenziert mit der GNU General Public 
 * License Version 3. Aktuelle Lizenzhinweise zu moodle 
 * finden Sie unter: https://moodledev.io/general/license.
 *
 * Das Plugin ist lizenziert mit der GNU General Public 
 * License Version 3. Anzugebender Urheber des Plugins ist
 * die „metromorph softworks GmbH im Auftrag von ORCA.nrw“.
 * Die Lizenzierung umfasst nicht die Inhalte, die auf der
 * Anwenderoberfläche des Plugins angezeigt werden; diese sind 
 * ihrerseits CC BY-SA 4.0 lizenziert.
 *
 * Sie sollten eine Kopie der GNU-Lizenz zusammen mit diesem 
 * Programm erhalten haben. Sie finden sie auch unter 
 * https://www.gnu.org/licenses. Die CC-Lizenz finden Sie
 * unter https://creativecommons.org/licenses/by-sa/4.0/deed.de.
 */

/**
 * Library of interface functions and constants.
 *
 * @package   mod_splernstrategien
 * @license   https://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */

defined('MOODLE_INTERNAL') || die();

/**
 * Return if the plugin supports $feature.
 *
 * @param string $feature Constant representing the feature.
 * @return true | null True if the feature is supported, null otherwise.
 */
function splernstrategien_supports($feature)
{
    switch ($feature) {
        case FEATURE_MOD_INTRO:
            return true;
        case FEATURE_COMPLETION_HAS_RULES:
            return true;
        case FEATURE_MOD_PURPOSE: 
            return MOD_PURPOSE_CONTENT;
        default:
            return null;
    }
}

/**
 * Saves a new instance of the splernstrategien into the database.
 *
 * Given an object containing all the necessary data, (defined by the form
 * in mod_form.php) this function will create a new instance and return the id
 * number of the instance.
 *
 * @param object $moduleinstance An object from the form.
 * @param mod_splernstrategien_mod_form $mform The form.
 * @return int The id of the newly inserted record.
 */
function splernstrategien_add_instance($moduleinstance, $mform = null)
{
    global $DB;

    $moduleinstance->timecreated = time();

    //var_dump($moduleinstance);

    $id = $DB->insert_record('splernstrategien', $moduleinstance);

    return $id;
}

/**
 * Updates an instance of the splernstrategien in the database.
 *
 * Given an object containing all the necessary data (defined in mod_form.php),
 * this function will update an existing instance with new data.
 *
 * @param object $moduleinstance An object from the form in mod_form.php.
 * @param mod_splernstrategien_mod_form $mform The form.
 * @return bool True if successful, false otherwise.
 */
function splernstrategien_update_instance($moduleinstance, $mform = null)
{
    global $DB;

    $moduleinstance->timemodified = time();
    $moduleinstance->id = $moduleinstance->instance;

    return $DB->update_record('splernstrategien', $moduleinstance);
}

/**
 * Removes an instance of the splernstrategien from the database.
 *
 * @param int $id Id of the module instance.
 * @return bool True if successful, false on failure.
 */
function splernstrategien_delete_instance($id)
{
    global $DB;

    $exists = $DB->get_record('splernstrategien', array('id' => $id));

    if (!$exists) {
        return false;
    }
    $cm = get_coursemodule_from_instance('splernstrategien', $id);
    \core_completion\api::update_completion_date_event($cm->id, 'splernstrategien', $id, null);

    try {
        $transaction = $DB->start_delegated_transaction();
        $DB->delete_records('splernstrategien_users', array('moodle_id' => $cm->id));
        $DB->delete_records('splernstrategien_answers', array('moodle_id' => $cm->id));
        $DB->delete_records('splernstrategien', array('id' => $id));
        $transaction->allow_commit();
    } catch (Exception $e) {
        $transaction->rollback($e);
        // The above rollback should throw and we will never get here.
        return false;
    }
    
    return true;
}

/**
 * Add a get_coursemodule_info function in case any instance wants to add 'extra' information
 * for the course (see resource).
 *
 * Given a course_module object, this function returns any "extra" information that may be needed
 * when printing this activity in a course listing.  See get_array_of_activities() in course/lib.php.
 *
 * @param stdClass $coursemodule The coursemodule object (record).
 * @return cached_cm_info An object on information that the courses
 *                        will know about (most noticeably, an icon).
 */
function splernstrategien_get_coursemodule_info($coursemodule)
{
    $result = new cached_cm_info();

    // Populate the custom completion rules as key => value pairs, but only if the completion mode is 'automatic'.
    if ($coursemodule->completion == COMPLETION_TRACKING_AUTOMATIC) {
        $rules = \mod_splernstrategien\completion\custom_completion::get_defined_custom_rules();
        $result->customdata['customcompletionrules'] = array_fill_keys($rules, true);
    }
    return $result;
}

/**
 * Mark the activity completed (if required) and trigger the course_module_viewed event.
 *
 * @param  stdClass $instance  splernstrategien object
 * @param  stdClass $course  course object
 * @param  stdClass $cm      course module object
 * @param  stdClass $context context object
 */
function splernstrategien_view($course, $cm)
{
    // Completion.
    if ($cm->completion == COMPLETION_TRACKING_AUTOMATIC) {
        $completion = new completion_info($course);
        $completion->update_state($cm);
    }
}
