<?php
/**
 * Diese Datei ist Teil von „Lernstrategien“ 
 * (Dateiname: „mod_splernstrategien“), einer Plugin-Software 
 * für das moodle LMS.
 *
 * Das moodle LMS ist lizenziert mit der GNU General Public 
 * License Version 3. Aktuelle Lizenzhinweise zu moodle 
 * finden Sie unter: https://moodledev.io/general/license.
 *
 * Das Plugin ist lizenziert mit der GNU General Public 
 * License Version 3. Anzugebender Urheber des Plugins ist
 * die „metromorph softworks GmbH im Auftrag von ORCA.nrw“.
 * Die Lizenzierung umfasst nicht die Inhalte, die auf der
 * Anwenderoberfläche des Plugins angezeigt werden; diese sind 
 * ihrerseits CC BY-SA 4.0 lizenziert.
 *
 * Sie sollten eine Kopie der GNU-Lizenz zusammen mit diesem 
 * Programm erhalten haben. Sie finden sie auch unter 
 * https://www.gnu.org/licenses. Die CC-Lizenz finden Sie
 * unter https://creativecommons.org/licenses/by-sa/4.0/deed.de.
 */

/**
 * Implementation for activity-completion.
 *
 * @package   mod_splernstrategien
 * @license   https://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */
namespace mod_splernstrategien;

require(__DIR__ . '/../../config.php');

global $PAGE, $DB, $_SESSION, $OUTPUT, $USER, $COURSE;
require_login();

$PAGE->set_context(\context_system::instance());
$PAGE->set_title('Lernstrategien');
$PAGE->set_heading('Lernstrategien');
$PAGE->set_cacheable(false);
$url = new \moodle_url("/mod/splernstrategien/view.php");
$PAGE->set_url($url);

$cm_id = optional_param('id', 0, PARAM_INT);
//get id from session if not provided as param
if (!$cm_id && array_key_exists("cm_id", $_SESSION)) {
    $cm_id = $_SESSION["cm_id"];
}

// Initialize user-entry in DB
// use user-id and course-id ($cm_id)
list($course, $cm) = get_course_and_cm_from_cmid($cm_id, 'splernstrategien');
$_SESSION["cm_id"] = $cm->id; //TODO: Session aufrufe vermeiden?
$_SESSION["course"] = $course->id;


// Get or init the LS user from current user, course and module.
$user = user::from_moodleuser($USER, $course->id, $cm->id);
if ($user === false) {
    $user = user::init($USER, $course->id, $cm->id);
}

// Admins can set state directly.
$newstate = NULL;
if (has_capability('moodle/site:config', \context_system::instance())) {
    $newstate = optional_param('state', NULL, PARAM_INT);
}

$fsm = new statemachine($user, $newstate);

if (is_null($newstate)) {
    // No new state given, calculate the transition.
    
    // \todo\refactor Dont just take all post data but only relevant ones for current state.
    $userinput = $_POST;
    if (is_null($userinput)) {
        $userinput = [];
    }

    $fsm->transition($userinput);
    
    $userinput = array_filter(
        $userinput,
        fn($key) => ($key != "state") && ($key != "type"),
        ARRAY_FILTER_USE_KEY
    );
    // \todo\security So we just store everything away to the DB with no bounds checking?
    // \todo Remove Session.
    $_SESSION["answers"] = (object) $user->merge_answers($userinput);
} else {
    // State explictilly set. Reset delay.
    $user->delay = 0;
}

// Has the user finished? Handle delay states.
$fsm->special_transitions();

lib::renderData($fsm);

// \todo\future We should save the data before rendering. This is not possible atm since there is
// legacy data changing code in the render routines.
$user->save();
